/*
 * FCKeditor - The text editor for Internet - http://www.fckeditor.net
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 * 
 * == BEGIN LICENSE ==
 * 
 * Licensed under the terms of any of the following licenses at your
 * choice:
 * 
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 * 
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 * 
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * == END LICENSE ==
 */
package net.fckeditor;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Properties;

import net.fckeditor.tool.Utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Static configuration handler object, which provides getters and setters for basic values like fckeditor dir, base dir
 * (user dir) and defaults for height, width and toolbar of the editor. All values are preset by the defaults defined in
 * default.properties. This object is used everywhere these values are needed. <br />
 * If the {@link IBaseDirProvider} is set, {@link #getBaseDir()} is calling {@link IBaseDirProvider#getDir()} to get the
 * user dir.
 * 
 * @version $Id:ConfigurationHandler.java 1099 2008-11-06 15:01:50Z th-schwarz $
 */
public class ConfigurationHandler {
	private static final Logger logger = LoggerFactory.getLogger(ConfigurationHandler.class);

	private static Properties defaultProperties = new Properties();
	private static String baseDir;
	private static IBaseDirProvider baseDirProvider = null;
	private static String fckEditorDir;
	private static String fckEditorHeight;
	private static String fckEditorWidth;
	private static String fckEditorToolbarSet;
	private static boolean forceSingleExtension;

	static {
		// load defaults
		try {
			defaultProperties.load(new BufferedInputStream(ConfigurationHandler.class
			    .getResourceAsStream("default.properties")));
		} catch (IOException e) {
			logger.error("Error while loading the default properties!", e);
			throw new RuntimeException("Can't load default properties!", e);
		}
		baseDir = defaultProperties.getProperty("fckeditor.basedir");
		fckEditorDir = defaultProperties.getProperty("fckeditor.dir");
		fckEditorWidth = defaultProperties.getProperty("fckeditor.width");
		fckEditorHeight = defaultProperties.getProperty("fckeditor.height");
		fckEditorToolbarSet = defaultProperties.getProperty("fckeditor.toolbarset");
		forceSingleExtension = Boolean.valueOf(defaultProperties.getProperty("fckeditor.forcesingleextension"));

		logger.info("Default properties loaded and initialized successfully.");
	}

	
	/**
	 * Getter for the base dir (using for user files).
	 * 
	 * @return If the {@link IBaseDirProvider} is set, then it's dir getter is called, or base dir.
	 */
	public static String getBaseDir() {
		return (baseDirProvider != null) ? baseDirProvider.getDir() : baseDir;
	}

	/**
	 * Setter for the base dir (using for user files).
	 * 
	 * @param baseDir
	 *          relative to the context root (no leading or ending /).
	 */
	public static void setBaseDir(final String baseDir) {
		ConfigurationHandler.baseDir = baseDir;
	}

	/**
	 * Getter for the {@link IBaseDirProvider}.
	 * 
	 * @param baseDirProvider
	 */
	public static void setBaseDirProvider(final IBaseDirProvider baseDirProvider) {
		ConfigurationHandler.baseDirProvider = baseDirProvider;
	}

	/**
	 * Getter for the dir of the fckeditor.
	 * 
	 * @return Dir of the fckeditor relative to the context root.
	 */
	public static String getFckEditorDir() {
		return fckEditorDir;
	}

	/**
	 * Setter for the dir of the fckeditor.
	 * 
	 * @param fckEditorDir
	 *          relative to the context root (no leading or ending /).
	 */
	public static void setFckEditorDir(final String fckEditorDir) {
		ConfigurationHandler.fckEditorDir = fckEditorDir;
	}

	/**
	 * Getter for the default height of the fckeditor.
	 * 
	 * @return Default height of the fckeditor.
	 */
	public static String getFckEditorHeight() {
		return fckEditorHeight;
	}

	/**
	 * Setter for the default height of the fckeditor.
	 * 
	 * @param fckEditorHeight
	 *          the default height of the fckeditor.
	 */
	public static void setFckEditorHeight(final String fckEditorHeight) {
		ConfigurationHandler.fckEditorHeight = fckEditorHeight;
	}

	/**
	 * Getter for the default width of the fckeditor.
	 * 
	 * @return Default width of the fckeditor.
	 */
	public static String getFckEditorWidth() {
		return fckEditorWidth;
	}

	/**
	 * Setter for the default width of the fckeditor.
	 * 
	 * @param fckEditorWidth
	 *          the default width of the fckeditor.
	 */
	public static void setFckEditorWidth(final String fckEditorWidth) {
		ConfigurationHandler.fckEditorWidth = fckEditorWidth;
	}

	/**
	 * Getter of the name of the default toolbarset of the fckeditor.
	 * 
	 * @return Name of the default toolbar set.
	 */
	public static String getFckEditorToolbarSet() {
		return fckEditorToolbarSet;
	}

	/**
	 * Getter of the name of the default toolbarset of the fckeditor.
	 * 
	 * @param fckEditorToolbarSet
	 *          the name of the toolbarset.
	 */
	public static void setFckEditorToolbarSet(final String fckEditorToolbarSet) {
		ConfigurationHandler.fckEditorToolbarSet = fckEditorToolbarSet;
	}
	
	/**
	 * Getter for the default handling of single extensions.
	 * 
   * @return the forceSingleExtension
   */
  public static boolean isForceSingleExtension() {
  	return forceSingleExtension;
  }

	/**
	 * Setter for the default handling of single extensions.
	 *
   * @param forceSingleExtension the forceSingleExtension to set
   */
  public static void setForceSingleExtension(boolean forceSingleExtension) {
  	ConfigurationHandler.forceSingleExtension = forceSingleExtension;
  }

  /**
   * Just a wrapper to {@link #setForceSingleExtension(boolean)}.
   * 
   * @param forceSingleExtension
   */
  public static void setForceSingleExtension(final String forceSingleExtension) {
  	ConfigurationHandler.forceSingleExtension = Boolean.parseBoolean(forceSingleExtension);
  }
  
	/**
	 * Getter for a default property with the name 'key'.
	 * 
	 * @param key
	 * @return The required property or null, if doesn't exists.
	 */
	public static String getDefaultProperty(final String key) {
		return defaultProperties.getProperty(key);
	}

	/**
	 * Getter for the dafault properties.
	 * 
	 * @return Default properties.
	 */
	public static Properties getDefaultProperties() {
		return defaultProperties;
	}

	/**
	 * Getter for the sub dir name of a media type.
	 * 
	 * @param type
	 * @return The sub dir name of the requested type, or the default one ('file') if 'type' is unknown or empty.
	 */
	public static String getSubDirForType(final String type) {
		String defaultSubDir = getDefaultProperty("fckeditor.subdir.media.file");
		if (Utils.isEmpty(type))
			return defaultSubDir;
		String subDir = getDefaultProperty("fckeditor.subdir.media.".concat(type.toLowerCase()));

		return (Utils.isEmpty(subDir)) ? defaultSubDir : subDir;
	}
}
